module RmPlugin
  #
  # Plugins for the removal of users.
  #

  def RmPlugin.included(c)
    # Callback, called whenever another class or module includes this
    # one. The parameter given is the name of the class or module
    # that included us.
    @includers ||= []
    @includers << c
  end

  def RmPlugin.includers
    return @includers
  end

  def runner()
    return RmRunner
  end

  def dummy_runner()
    return RmDummyRunner
  end

  def delete_domain(domain)
    # Delete the given domain. Some plugins don't have a concept of
    # domains, so just delete all users with a username that looks
    # like it's in the given domain.
    usernames = list_domains_users([domain])

    raise NonexistentDomainError.new(domain) if usernames.empty?

    usernames.each do |u|
      delete_user(u)
    end
  end

  def delete_user(user)
    # Delete the given user.
    raise NotImplementedError
  end

end
